@@ -15,7 +15,6 @@ service_info_map = {
 }
 
 api_info = {
-
     # 添加加速域名: https://www.volcengine.com/docs/6454/97340
     "AddCdnDomain": ApiInfo("POST", "/", {
         "Action": "AddCdnDomain", "Version": SERVICE_VERSION}, {}, {}),
@@ -92,10 +91,6 @@ api_info = {
     "DescribeCdnRegionAndIsp": ApiInfo("POST", "/", {
         "Action": "DescribeCdnRegionAndIsp", "Version": SERVICE_VERSION}, {}, {}),
 
-    # （Deprecated）查询域名排行数据: https://www.volcengine.com/docs/6454/70447?type=preview
-    "DescribeCdnDomainTopData": ApiInfo("POST", "/", {
-        "Action": "DescribeCdnDomainTopData", "Version": SERVICE_VERSION}, {}, {}),
-
     # 查询账号计费方式: https://www.volcengine.com/docs/6454/78999
     "DescribeCdnService": ApiInfo("POST", "/", {
         "Action": "DescribeCdnService", "Version": SERVICE_VERSION}, {}, {}),
@@ -184,6 +179,7 @@ api_info = {
     "BatchDeployCert": ApiInfo("POST", "/", {
         "Action": "BatchDeployCert", "Version": SERVICE_VERSION}, {}, {}),
 
+
 }
 
 
@@ -403,16 +399,6 @@ class CDNService(Service):
         res_json = json.loads(res)
         return res_json
 
-    def describe_cdn_domain_top_data(self, params=None):
-        if params is None:
-            params = {}
-        action = "DescribeCdnDomainTopData"
-        res = self.json(action, [], json.dumps(params))
-        if res == '':
-            raise Exception("%s: empty response" % action)
-        res_json = json.loads(res)
-        return res_json
-
     def describe_cdn_service(self, params=None):
         if params is None:
             params = {}
