@@ -10,116 +10,21 @@ from volcengine.base.Service import Service
 from volcengine.const.Const import *
 from volcengine.util.Util import *
 from volcengine.Policy import *
-
-IMAGEX_SERVICE_NAME = "ImageX"
-IMAGEX_API_VERSION = "2018-08-01"
-
-ResourceServiceIdTRN = "trn:ImageX:*:*:ServiceId/%s"
-ResourceStoreKeyTRN = "trn:ImageX:*:*:StoreKeys/%s"
-
-UPLOAD_THREADS = 3
-
-service_info_map = {
-    REGION_CN_NORTH1: ServiceInfo(
-        "imagex.volcengineapi.com",
-        {'Accept': 'application/json'},
-        Credentials('', '', IMAGEX_SERVICE_NAME, REGION_CN_NORTH1),
-        10, 10, "https"),
-    REGION_AP_SINGAPORE1: ServiceInfo(
-        "imagex-ap-singapore-1.volcengineapi.com",
-        {'Accept': 'application/json'},
-        Credentials('', '', IMAGEX_SERVICE_NAME, REGION_AP_SINGAPORE1),
-        10, 10, "https"),
-    REGION_US_EAST1: ServiceInfo(
-        "imagex-us-east-1.volcengineapi.com",
-        {'Accept': 'application/json'},
-        Credentials('', '', IMAGEX_SERVICE_NAME, REGION_US_EAST1),
-        10, 10, "https"),
-}
-
-api_info = {
-    # 模板管理
-    "CreateImageTemplate":
-        ApiInfo("POST", "/", {"Action": "CreateImageTemplate",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "DeleteImageTemplate":
-        ApiInfo("POST", "/", {"Action": "DeleteImageTemplate",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "PreviewImageTemplate":
-        ApiInfo("POST", "/", {"Action": "PreviewImageTemplate",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageTemplate":
-        ApiInfo("GET", "/", {"Action": "GetImageTemplate",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetAllImageTemplates":
-        ApiInfo("GET", "/", {"Action": "GetAllImageTemplates",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    # 资源管理相关
-    "ApplyImageUpload":
-        ApiInfo("GET", "/", {"Action": "ApplyImageUpload",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "CommitImageUpload":
-        ApiInfo("POST", "/", {"Action": "CommitImageUpload",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "DeleteImageUploadFiles":
-        ApiInfo("POST", "/", {"Action": "DeleteImageUploadFiles",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "UpdateImageUploadFiles":
-        ApiInfo("POST", "/", {"Action": "UpdateImageUploadFiles",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "PreviewImageUploadFile":
-        ApiInfo("GET", "/", {"Action": "PreviewImageUploadFile",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageUploadFile":
-        ApiInfo("GET", "/", {"Action": "GetImageUploadFile",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageUploadFiles":
-        ApiInfo("GET", "/", {"Action": "GetImageUploadFiles",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageUpdateFiles":
-        ApiInfo("GET", "/", {"Action": "GetImageUpdateFiles",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageOCR":
-        ApiInfo("POST", "/", {"Action": "GetImageOCR",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetSegmentImage":
-        ApiInfo("POST", "/", {"Action": "GetSegmentImage",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageQuality":
-        ApiInfo("POST", "/", {"Action": "GetImageQuality",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageEraseModels":
-        ApiInfo("GET", "/", {"Action": "GetImageEraseModels",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageEraseResult":
-        ApiInfo("POST", "/", {"Action": "GetImageEraseResult",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageEnhanceResult":
-        ApiInfo("POST", "/", {"Action": "GetImageEnhanceResult",
-                              "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageBgFillResult":
-        ApiInfo("POST", "/", {"Action": "GetImageBgFillResult",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageComicResult":
-        ApiInfo("POST", "/", {"Action": "GetImageComicResult",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageSuperResolutionResult":
-        ApiInfo("POST", "/", {"Action": "GetImageSuperResolutionResult",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetImageSmartCropResult":
-        ApiInfo("POST", "/", {"Action": "GetImageSmartCropResult",
-                "Version": IMAGEX_API_VERSION}, {}, {}),
-    "GetLicensePlateDetection":
-        ApiInfo("POST", "/", {"Action": "GetLicensePlateDetection",
-                "Version": IMAGEX_API_VERSION}, {}, {})
-}
+from volcengine.imagex.ImageXConfig import *
+from retry import retry
 
 
 class Uploader:
-    def __init__(self, host, store_infos=[], datas=[]):
+    def __init__(self, imagex_service, host, store_infos=None, file_paths_or_bytes=None):
+        if store_infos is None:
+            store_infos = []
+        if file_paths_or_bytes is None:
+            file_paths_or_bytes = []
+
+        self.imagex_service = imagex_service
         self.host = host
         self.store_infos = store_infos
-        self.datas = datas
+        self.file_paths_or_bytes = file_paths_or_bytes
 
         self.queue = queue.Queue()
         self.queue_lock = threading.Lock()
@@ -135,19 +40,109 @@ class Uploader:
                 break
             idx = self.queue.get()
             self.queue_lock.release()
-            self.upload_by_host(idx)
 
-    def upload_by_host(self, idx):
-        store_info = self.store_infos[idx]
-        img_data = self.datas[idx]
+            store_info = self.store_infos[idx]
+            file_paths_or_bytes = self.file_paths_or_bytes[idx]
+
+            if isinstance(file_paths_or_bytes, bytes):
+                self.upload_by_host(store_info, file_paths_or_bytes)
+            elif isinstance(file_paths_or_bytes, str):
+                file_path = file_paths_or_bytes
+                file_size = os.path.getsize(file_path)
+                data = open(file_path, "rb")
+                if file_size < MinChunkSize:
+                    self.upload_by_host(store_info, data.read())
+                elif file_size > LargeFileSize:
+                    self.chunk_upload(store_info, data, file_size, True)
+                else:
+                    self.chunk_upload(store_info, data, file_size, False)
+                data.close()
+            else:
+                raise Exception("Uploader only accept bytes or path data")
+
+    @retry(tries=3, delay=1, backoff=2)
+    def upload_by_host(self, store_info, img_data):
         url = 'https://{}/{}'.format(self.host, store_info['StoreUri'])
         check_sum = crc32(img_data) & 0xFFFFFFFF
         check_sum = "%08x" % check_sum
         headers = {'Content-CRC32': check_sum, 'Authorization': store_info['Auth']}
-        upload_status, resp = ImageXService().put_data(url, img_data, headers)
+        upload_status, resp = self.imagex_service.put_data(url, img_data, headers)
         if not upload_status:
             raise Exception('upload by host: upload url %s status false, resp: %s' % (url, resp))
 
+    def chunk_upload(self, store_info, f, size, is_large_file):
+        upload_id = self.init_upload_part(store_info, is_large_file)
+        n = size // MinChunkSize
+        last_num = n - 1
+        parts = []
+        for i in range(0, last_num):
+            data = f.read(MinChunkSize)
+            part_number = i
+            if is_large_file:
+                part_number = i + 1
+            part = self.upload_part(store_info, upload_id, part_number, data, is_large_file)
+            parts.append(part)
+        data = f.read()
+        if is_large_file:
+            last_num = last_num + 1
+        part = self.upload_part(store_info, upload_id, last_num, data, is_large_file)
+        parts.append(part)
+        return self.upload_merge_part(store_info, upload_id, parts, is_large_file)
+
+    @retry(tries=3, delay=1, backoff=2)
+    def init_upload_part(self, store_info, is_large_file):
+        url = 'https://{}/{}?uploads'.format(self.host, store_info['StoreUri'])
+        headers = {'Authorization': store_info['Auth']}
+        if is_large_file:
+            headers['X-Storage-Mode'] = 'gateway'
+        upload_status, resp = self.imagex_service.put_data(url, None, headers)
+        resp = json.loads(resp)
+        if not upload_status:
+            raise Exception("init upload error")
+        if resp.get('success') is None or resp['success'] != 0:
+            raise Exception("init upload error")
+        return resp['payload']['uploadID']
+
+    @retry(tries=3, delay=1, backoff=2)
+    def upload_part(self, store_info, upload_id, part_number, data, is_large_file):
+        url = 'https://{}/{}?partNumber={}&uploadID={}'.format(self.host, store_info['StoreUri'], part_number, upload_id)
+        check_sum = crc32(data) & 0xFFFFFFFF
+        check_sum = "%08x" % check_sum
+        headers = {'Content-CRC32': check_sum, 'Authorization': store_info['Auth']}
+        if is_large_file:
+            headers['X-Storage-Mode'] = 'gateway'
+        upload_status, resp = self.imagex_service.put_data(url, data, headers)
+        if not upload_status:
+            raise Exception(url + json.dumps(resp))
+        resp = json.loads(resp)
+        if resp.get('success') is None or resp['success'] != 0:
+            raise Exception("upload part error")
+        return check_sum
+
+    # noinspection DuplicatedCode
+    @staticmethod
+    def generate_merge_body(check_sum_list):
+        if len(check_sum_list) == 0:
+            raise Exception('crc32 list empty')
+        s = []
+        for i in range(len(check_sum_list)):
+            s.append('{}:{}'.format(i, check_sum_list[i]))
+        comma = ','
+        return comma.join(s)
+
+    @retry(tries=3, delay=1, backoff=2)
+    def upload_merge_part(self, store_info, upload_id, check_sum_list, is_large_file):
+        url = 'https://{}/{}?uploadID={}'.format(self.host, store_info['StoreUri'], upload_id)
+        data = self.generate_merge_body(check_sum_list)
+        headers = {'Authorization': store_info['Auth']}
+        if is_large_file:
+            headers['X-Storage-Mode'] = 'gateway'
+        upload_status, resp = self.imagex_service.put_data(url, data, headers)
+        resp = json.loads(resp)
+        if not upload_status:
+            raise Exception("init upload error")
+        if resp.get('success') is None or resp['success'] != 0:
+            raise Exception("init upload error")
 
 class ImageXService(Service):
     def __init__(self, region=REGION_CN_NORTH1):
@@ -182,17 +177,51 @@ class ImageXService(Service):
         return res_json
 
     # upload local image file
+    # noinspection DuplicatedCode
     def upload_image(self, params, file_paths):
-        img_datas = []
         for p in file_paths:
             if not os.path.isfile(p):
                 raise Exception("no such file on file path %s" % p)
-            in_file = open(p, "rb")
-            img_datas.append(in_file.read())
-            in_file.close()
-        return self.upload_image_data(params, img_datas)
+
+        apply_upload_request = {
+            'ServiceId': params['ServiceId'],
+            'SessionKey': params.get('SessionKey', ''),
+            'UploadNum': len(file_paths),
+            'StoreKeys': params.get('StoreKeys', []),
+        }
+        resp = self.apply_upload(apply_upload_request)
+        if 'Error' in resp['ResponseMetadata']:
+            raise Exception(resp['ResponseMetadata'])
+
+        result = resp['Result']
+        reqid = result['RequestId']
+        addr = result['UploadAddress']
+        if len(addr['UploadHosts']) == 0:
+            raise Exception("no upload host found, reqid %s" % reqid)
+        elif len(addr['StoreInfos']) != len(file_paths):
+            raise Exception("store info len %d != upload num %d, reqid %s" % (
+                len(result['StoreInfos']), len(file_paths), reqid))
+
+        session_key = addr['SessionKey']
+        host = addr['UploadHosts'][0]
+        self.do_upload(file_paths, host, addr['StoreInfos'])
+
+        commit_upload_request = {
+            'ServiceId': params['ServiceId'],
+        }
+        commit_upload_body = {
+            'SessionKey': session_key,
+            'Functions': params.get('Functions', []),
+            'OptionInfos': params.get('OptionInfos', [])
+        }
+        resp = self.commit_upload(
+            commit_upload_request, json.dumps(commit_upload_body))
+        if 'Error' in resp['ResponseMetadata']:
+            raise Exception(resp['ResponseMetadata'])
+        return resp['Result']
 
     # upload image data
+    # noinspection DuplicatedCode
     def upload_image_data(self, params, img_datas):
         for data in img_datas:
             if not isinstance(data, bytes):
@@ -235,9 +264,9 @@ class ImageXService(Service):
             raise Exception(resp['ResponseMetadata'])
         return resp['Result']
 
-    def do_upload(self, img_datas, host, store_infos):
+    def do_upload(self, file_paths_or_bytes, host, store_infos):
         threads = []
-        uploader = Uploader(host, store_infos, img_datas)
+        uploader = Uploader(self, host, store_infos, file_paths_or_bytes)
         for i in range(UPLOAD_THREADS):
             thread = threading.Thread(target=uploader.async_upload)
             thread.start()
@@ -258,7 +287,7 @@ class ImageXService(Service):
         else:
             return base64.b64encode(data.decode('utf-8'))
 
-    def get_upload_auth(self, service_ids, expire=60*60, key_ptn=''):
+    def get_upload_auth(self, service_ids, expire=60 * 60, key_ptn=''):
         apply_res = []
         commit_res = []
         if len(service_ids) == 0:
@@ -359,7 +388,7 @@ class ImageXService(Service):
     def get_image_quality(self, params):
         body = {
             'ImageUrl': params['ImageUrl'],
-            'VqType':  params.get('VqType', None)
+            'VqType': params.get('VqType', None)
         }
         res = self.imagex_post('GetImageQuality', params, json.dumps(body))
         if res == '':
@@ -449,4 +478,4 @@ class ImageXService(Service):
         if res == '':
             raise Exception("%s: empty response" % 'GetLicensePlateDetection')
         res_json = json.loads(res)
-        return res_json
\ No newline at end of file
+        return res_json
